# Stripe Python bindings
# API docs at http://stripe.com/docs/api
# Authors:
# Patrick Collison <patrick@stripe.com>
# Greg Brockman <gdb@stripe.com>
# Andrew Metcalf <andrew@stripe.com>

# Configuration variables

api_key = None
api_base = 'https://api.stripe.com'
upload_api_base = 'https://uploads.stripe.com'
api_version = None
verify_ssl_certs = True

# Resource
from stripe.resource import (  # noqa
    Account, Balance, BalanceTransaction, BankAccount, Card,
    Charge, Customer, Invoice, InvoiceItem, Plan, Token, Coupon,
    Event, Transfer, Recipient, FileUpload,
    ApplicationFee, Subscription, BitcoinReceiver, BitcoinTransaction)

# Error imports.  Note that we may want to move these out of the root
# namespace in the future and you should prefer to access them via
# the fully qualified `stripe.error` module.

from stripe.error import (  # noqa
    StripeError, APIError, APIConnectionError, AuthenticationError, CardError,
    InvalidRequestError)

# DEPRECATED: These imports will be moved out of the root stripe namespace
# in version 2.0

from stripe.version import VERSION  # noqa
from stripe.api_requestor import APIRequestor  # noqa
from stripe.resource import (  # noqa
    convert_to_stripe_object, StripeObject, StripeObjectEncoder,
    APIResource, ListObject, SingletonAPIResource, ListableAPIResource,
    CreateableAPIResource, UpdateableAPIResource, DeletableAPIResource)
from stripe.util import json, logger  # noqa



# Exceptions
class StripeError(Exception):

    def __init__(self, message=None, http_body=None, http_status=None,
                 json_body=None, headers=None):
        super(StripeError, self).__init__(message)

        if http_body and hasattr(http_body, 'decode'):
            try:
                http_body = http_body.decode('utf-8')
            except:
                http_body = ('<Could not decode body as utf-8. '
                             'Please report to support@stripe.com>')

        self.http_body = http_body
        self.http_status = http_status
        self.json_body = json_body
        self.headers = headers or {}
        self.request_id = self.headers.get('request-id', None)

    def __str__(self):
        msg = super(StripeError, self).__str__()
        if self.request_id is not None:
            return "Request {}: {}".format(self.request_id, msg)
        else:
            return msg


class APIError(StripeError):
    pass


class APIConnectionError(StripeError):
    pass


class CardError(StripeError):

    def __init__(self, message, param, code, http_body=None,
                 http_status=None, json_body=None, headers=None):
        super(CardError, self).__init__(
            message, http_body, http_status, json_body,
            headers)
        self.param = param
        self.code = code


class InvalidRequestError(StripeError):

    def __init__(self, message, param, http_body=None,
                 http_status=None, json_body=None, headers=None):
        super(InvalidRequestError, self).__init__(
            message, http_body, http_status, json_body,
            headers)
        self.param = param


class AuthenticationError(StripeError):
    pass



import re
import sys

from stripe.multipart_data_generator import MultipartDataGenerator
from stripe.test.helper import StripeTestCase


class MultipartDataGeneratorTests(StripeTestCase):

    def test_generate_simple_multipart_data(self):
        test_file = open(__file__)

        params = {
            "key1": "value1",
            "key2": "value2",
            "key3": test_file
            }
        generator = MultipartDataGenerator()
        generator.add_params(params)
        post_data = generator.get_post_data()

        if sys.version_info < (3, 0):
            http_body = "".join([line for line in post_data])
        else:
            byte_array = bytearray([i for i in post_data])
            http_body = byte_array.decode('utf-8')

        self.assertTrue(re.search(
            r"Content-Disposition: form-data; name=\"key1\"", http_body))
        self.assertTrue(re.search(
            r"Content-Disposition: form-data; name=\"key2\"", http_body))
        self.assertTrue(re.search(
            r"Content-Disposition: form-data; name=\"key3\"; filename=\".+\"",
            http_body))
        self.assertTrue(re.search(
            r"Content-Type: application/octet-stream", http_body))

        test_file.seek(0)
        file_contents = test_file.read()
        self.assertNotEqual(-1, http_body.find(file_contents))
